module Degenz.Embeds

open DSharpPlus.EventArgs
open Degenz.Types
open DSharpPlus.Entities
open AsciiTableFormatter

let hackGif = "https://s10.gifyu.com/images/Hacker-Degenz-V2.gif"
let shieldGif = "https://s10.gifyu.com/images/Defense-Degenz-V2.gif"

let getHackGif = function
    | HackId.Virus -> "https://s10.gifyu.com/images/Attack-DegenZ.gif"
    | HackId.RemoteAccess -> "https://s10.gifyu.com/images/Mind-Control-Degenz-V2.gif"
    | HackId.Worm -> "https://s10.gifyu.com/images/WormBugAttack_Degenz.gif"
    | _ -> hackGif

let getShieldGif = function
    | ShieldId.Firewall -> "https://s10.gifyu.com/images/Defense-GIF-1-Degenz.gif"
    | ShieldId.Encryption -> "https://s10.gifyu.com/images/Encryption-Degenz-V2.gif"
    | ShieldId.Cypher -> "https://s10.gifyu.com/images/Matrix_Degenz.gif"
    | _ -> shieldGif


let constructEmbed message =
    let builder = DiscordEmbedBuilder()
    builder.Color <- Optional(DiscordColor.Blurple)
    builder.Description <- message
    let author = DiscordEmbedBuilder.EmbedAuthor()
    author.Name <- "Degenz Hacker Game"
    author.Url <- "https://twitter.com/degenzgame"
    author.IconUrl <- "https://pbs.twimg.com/profile_images/1473192843359309825/cqjm0VQ4_400x400.jpg"
    builder.Author <- author
    builder.Build()

let pickDefense actionId player =
    let buttons =
        Messaging.constructButtons actionId (string player.DiscordId) (Player.shields player)
        |> Seq.cast<DiscordComponent>

    let embed =
        DiscordEmbedBuilder()
            .WithColor(DiscordColor.Blurple)
            .WithDescription("Pick a defense to mount for 10 hours")
            .WithImageUrl(shieldGif)

    DiscordInteractionResponseBuilder()
        .AddComponents(buttons)
        .AddEmbed(embed)
        .AsEphemeral true

let pickHack actionId attacker defender =
    let buttons =
        Messaging.constructButtons actionId $"{defender.DiscordId}-{defender.Name}" (Player.hacks attacker)
        |> Seq.cast<DiscordComponent>

    let embed =
        DiscordEmbedBuilder()
            .WithColor(DiscordColor.Blurple)
            .WithDescription("Pick the hack that you want to use")
            .WithImageUrl(hackGif)

    DiscordInteractionResponseBuilder()
        .AddComponents(buttons)
        .AddEmbed(embed.Build())
        .AsEphemeral true

let responseSuccessfulHack defenderName hack prize =
    let embed = DiscordEmbedBuilder()
    embed.ImageUrl <- getHackGif hack

    DiscordInteractionResponseBuilder()
        .WithContent($"Successfully hacked {defenderName} using {hack}! You just won {prize} GoodBoyTokenz!")
        .AddEmbed(embed.Build())
        .AsEphemeral(true)

let responseSuccessfulHackTrainer (hack : BattleItem) =
    let embed = DiscordEmbedBuilder()
    embed.ImageUrl <- getHackGif (enum<HackId>(hack.Id))

    DiscordFollowupMessageBuilder()
        .WithContent($"You successfully hacked <@{GuildEnvironment.botHackerBattle}> using {hack.Name}, and stole 💰$GBT {Game.HackPrize} from them!")
        .AddEmbed(embed.Build())
        .AsEphemeral(true)

let responseCreatedShield shield =
    DiscordInteractionResponseBuilder()
        .AddEmbed(DiscordEmbedBuilder().WithImageUrl(getShieldGif shield))
        .AsEphemeral(true)
        .WithContent($"Mounted a {shield} defense for 6 hours")

let responseCreatedShieldTrainer (shield : BattleItem) =
    DiscordFollowupMessageBuilder()
        .AddEmbed(DiscordEmbedBuilder().WithImageUrl(getShieldGif (enum<ShieldId>(shield.Id))))
        .AsEphemeral(true)
        .WithContent($"Mounted a {shield.Name} defense for 6 hours")

let eventSuccessfulHack (event : ComponentInteractionCreateEventArgs) targetId prize =
    DiscordMessageBuilder()
        .WithContent($"{event.User.Username} successfully hacked <@{targetId}> for a total of {prize} GoodBoyTokenz")

let eventFailedHack (event : ComponentInteractionCreateEventArgs) targetId prize =
//    let embed =
//        DiscordEmbedBuilder()
//            .WithColor(DiscordColor.Blurple)
//            .WithDescription("Pick the hack that you want to use")
//            .WithImageUrl(hackGif)
//
    DiscordMessageBuilder()
        .WithContent($"{event.User.Username} successfully hacked <@{targetId}> for a total of {prize} GoodBoyTokenz")

[<CLIMutable>]
type Table = {
    Name : string
    Cost : string
    Class : string
}

let storeListing store =
    let embeds =
        store
        |> Array.groupBy (fun (bi : BattleItem) -> bi.Type)
        |> Array.map (fun ( itemType , items ) ->
            items
            |> Array.map (fun item -> { Name = item.Name ; Cost = string item.Cost ; Class = string item.Class })
            |> Formatter.Format
            |> sprintf "**%As**\n``` %s ```" itemType
            |> constructEmbed)

    DiscordInteractionResponseBuilder()
        .AddEmbeds(embeds)
        .AsEphemeral(true)
